@@ -1,7 +1,7 @@
 /* grdesktop - gtk rdesktop frontend
  * Copyright (C) 2002 Thorsten Sauter <tsauter@gmx.net>
  *
- * $grdesktop: grdesktop/src/optbox.c,v 1.66 2003/11/07 12:41:20 tsauter Exp $
+ * $grdesktop: grdesktop/src/optbox.c,v 1.69 2003/11/17 16:21:01 tsauter Exp $
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -43,7 +43,7 @@ GtkWidget *option_box() {
 	gtk_notebook_insert_page(GTK_NOTEBOOK(note), page2, cpage2, -1);
 	gtk_widget_show(page2);
 	
-	cpage3 = gtk_label_new(_("Keyboard"));
+	cpage3 = gtk_label_new(_("Resources"));
 	page3 = gtk_table_new(1, 1, FALSE);
 	gtk_container_border_width(GTK_CONTAINER(page3), 5);
 	gtk_notebook_insert_page(GTK_NOTEBOOK(note), page3, cpage3, -1);
@@ -301,63 +301,108 @@ GtkWidget *options_page2_box() {
 GtkWidget *options_page3_box() {
 	GtkWidget *box;
 	GtkWidget *tbox;
-	GtkWidget *frame1;
-	GtkWidget *uTable;
-	GtkWidget *vboxRight, *vboxLeft;
-	GtkWidget *image;
-	GtkWidget *topBox;
-	GtkWidget *topLabel;
-	GtkWidget *tbl1;
+	GtkWidget *frame1, *frame2;
+	GtkWidget *uTable1, *uTable2;
+	GtkWidget *vboxRight1, *vboxLeft1;
+	GtkWidget *vboxRight2, *vboxLeft2;
+	GtkWidget *image1, *image2;
+	GtkWidget *topBox1, *topBox2;
+	GtkWidget *topLabel1, *topLabel2;
+	GtkWidget *tbl1, *tbl2;
 
 	box = gtk_vbox_new(FALSE, 10);
-	tbox = gtk_hbox_new(FALSE, 0);
+	tbox = gtk_vbox_new(FALSE, 0);
 	gtk_box_pack_start(GTK_BOX(box), tbox, FALSE, FALSE, FALSE);
 	gtk_widget_show(tbox);
 
-	/* create program settings box */
+	/* create keyboard settings box */
 	frame1 = gtk_frame_new(_("Keyboard type"));
 	gtk_container_add(GTK_CONTAINER(tbox), frame1);
 	gtk_widget_show(frame1);
 
-	uTable = gtk_table_new(1, 2, FALSE);
-	gtk_container_border_width(GTK_CONTAINER(uTable), 0);
-	gtk_container_add(GTK_CONTAINER(frame1), uTable);
-	gtk_widget_show(uTable);
+	uTable1 = gtk_table_new(1, 2, FALSE);
+	gtk_container_border_width(GTK_CONTAINER(uTable1), 0);
+	gtk_container_add(GTK_CONTAINER(frame1), uTable1);
+	gtk_widget_show(uTable1);
 
 	/* create the left image container */
-	vboxLeft = gtk_vbox_new(FALSE, 0);
-	gtk_container_set_border_width(GTK_CONTAINER(vboxLeft), 0);
-	gtk_table_attach(GTK_TABLE(uTable), vboxLeft, 0, 1, 0, 1,
+	vboxLeft1 = gtk_vbox_new(FALSE, 0);
+	gtk_container_set_border_width(GTK_CONTAINER(vboxLeft1), 0);
+	gtk_table_attach(GTK_TABLE(uTable1), vboxLeft1, 0, 1, 0, 1,
 		GTK_SHRINK, GTK_EXPAND|GTK_FILL, 5, 0);
-	gtk_widget_show(vboxLeft);
+	gtk_widget_show(vboxLeft1);
 
-	image = gtk_image_new_from_file(PIXDIR"/keyboard.png");
-	gtk_box_pack_start(GTK_BOX(vboxLeft), image, FALSE, FALSE, FALSE);
-	gtk_widget_show(image);
+	image1 = gtk_image_new_from_file(PIXDIR"/keyboard.png");
+	gtk_box_pack_start(GTK_BOX(vboxLeft1), image1, FALSE, FALSE, FALSE);
+	gtk_widget_show(image1);
 
 	/* create the main vbox to store components */
-	vboxRight = gtk_vbox_new(FALSE, 0);
-	gtk_container_set_border_width(GTK_CONTAINER(vboxRight), 0);
-	gtk_table_attach(GTK_TABLE(uTable), vboxRight, 1, 2, 0, 1,
+	vboxRight1 = gtk_vbox_new(FALSE, 0);
+	gtk_container_set_border_width(GTK_CONTAINER(vboxRight1), 0);
+	gtk_table_attach(GTK_TABLE(uTable1), vboxRight1, 1, 2, 0, 1,
 		GTK_EXPAND|GTK_FILL, GTK_EXPAND|GTK_FILL, 5, 5);
-	gtk_widget_show(vboxRight);
+	gtk_widget_show(vboxRight1);
 
 	/* create box for the top label */
-	topBox = gtk_hbox_new(FALSE, 0);
-	gtk_box_pack_start(GTK_BOX(vboxRight), topBox, FALSE, FALSE, FALSE);
-	gtk_widget_show(topBox);
+	topBox1 = gtk_hbox_new(FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(vboxRight1), topBox1, FALSE, FALSE, FALSE);
+	gtk_widget_show(topBox1);
 
-	topLabel = gtk_label_new(_("Select the language code for\nthe keyboard."));
-	gtk_box_pack_start(GTK_BOX(topBox), topLabel, FALSE, FALSE, FALSE);
-	gtk_widget_show(topLabel);
+	topLabel1 = gtk_label_new(_("Select the language code for\nthe keyboard."));
+	gtk_box_pack_start(GTK_BOX(topBox1), topLabel1, FALSE, FALSE, FALSE);
+	gtk_widget_show(topLabel1);
 
 	/* create a table for the main components and fill it */
 	tbl1 = gtk_table_new(1, 2, FALSE);
 	gtk_container_border_width(GTK_CONTAINER(tbl1), 10);
-	gtk_box_pack_start(GTK_BOX(vboxRight), tbl1, FALSE, FALSE, TRUE);
+	gtk_box_pack_start(GTK_BOX(vboxRight1), tbl1, FALSE, FALSE, TRUE);
 	gtk_widget_show(tbl1);
 	insert_keyboard(tbl1);
 
+	/* create sound settings box */
+	frame2 = gtk_frame_new(_("Sound on the remote Computer"));
+	gtk_container_add(GTK_CONTAINER(tbox), frame2);
+	gtk_widget_show(frame2);
+
+	uTable2 = gtk_table_new(1, 2, FALSE);
+	gtk_container_border_width(GTK_CONTAINER(uTable2), 0);
+	gtk_container_add(GTK_CONTAINER(frame2), uTable2);
+	gtk_widget_show(uTable2);
+
+	/* create the left image container */
+	vboxLeft2 = gtk_vbox_new(FALSE, 0);
+	gtk_container_set_border_width(GTK_CONTAINER(vboxLeft2), 0);
+	gtk_table_attach(GTK_TABLE(uTable2), vboxLeft2, 0, 1, 0, 1,
+		GTK_SHRINK, GTK_EXPAND|GTK_FILL, 5, 0);
+	gtk_widget_show(vboxLeft2);
+
+	image2 = gtk_image_new_from_file(PIXDIR"/sound.png");
+	gtk_box_pack_start(GTK_BOX(vboxLeft2), image2, FALSE, FALSE, FALSE);
+	gtk_widget_show(image2);
+
+	/* create the main vbox to store components */
+	vboxRight2 = gtk_vbox_new(FALSE, 0);
+	gtk_container_set_border_width(GTK_CONTAINER(vboxRight2), 0);
+	gtk_table_attach(GTK_TABLE(uTable2), vboxRight2, 1, 2, 0, 1,
+		GTK_EXPAND|GTK_FILL, GTK_EXPAND|GTK_FILL, 5, 5);
+	gtk_widget_show(vboxRight2);
+
+	/* create box for the top label */
+	topBox2 = gtk_hbox_new(FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(vboxRight2), topBox2, FALSE, FALSE, FALSE);
+	gtk_widget_show(topBox2);
+
+	topLabel2 = gtk_label_new(_("Please select which sound output you\nwould like to use."));
+	gtk_box_pack_start(GTK_BOX(topBox2), topLabel2, FALSE, FALSE, FALSE);
+	gtk_widget_show(topLabel2);
+
+	/* create a table for the main components and fill it */
+	tbl2 = gtk_table_new(1, 2, FALSE);
+	gtk_container_border_width(GTK_CONTAINER(tbl2), 10);
+	gtk_box_pack_start(GTK_BOX(vboxRight2), tbl2, FALSE, FALSE, TRUE);
+	gtk_widget_show(tbl2);
+	insert_sound(tbl2);
+
 	return(box);
 }
 
@@ -676,16 +721,16 @@ void insert_colorsel(const GtkWidget *widget) {
 	GtkWidget *menu;
 	gint i, count;
 
-	colorsize_menu = gtk_option_menu_new();
-	gtk_table_attach(GTK_TABLE(widget), colorsize_menu, 0, 1, 0, 1,
+	menu_colorsize = gtk_option_menu_new();
+	gtk_table_attach(GTK_TABLE(widget), menu_colorsize, 0, 1, 0, 1,
 		GTK_SHRINK, GTK_SHRINK, 0, 5);
-	gtk_widget_show(colorsize_menu);
+	gtk_widget_show(menu_colorsize);
 	menu = gtk_menu_new();
 
-	colorsize_img = gtk_image_new();
-	gtk_table_attach(GTK_TABLE(widget), colorsize_img, 0, 1, 1, 2,
+	image_colorsize = gtk_image_new();
+	gtk_table_attach(GTK_TABLE(widget), image_colorsize, 0, 1, 1, 2,
 		GTK_SHRINK, GTK_SHRINK, 0, 5);
-	gtk_widget_show(colorsize_img);
+	gtk_widget_show(image_colorsize);
 
 	count = g_list_length(colors);
 	for(i=0; i<count; i++) {
@@ -702,12 +747,12 @@ void insert_colorsel(const GtkWidget *widget) {
 		gtk_signal_connect(GTK_OBJECT(menuitem), "activate",
 			GTK_SIGNAL_FUNC(sig_colchange), NULL);
 	}
-	gtk_option_menu_set_menu(GTK_OPTION_MENU(colorsize_menu), menu);
+	gtk_option_menu_set_menu(GTK_OPTION_MENU(menu_colorsize), menu);
 
 	if(iSHASH("rdp_protocol") < 1) { /* disable, it's not supported by the RDP version */
-		gtk_widget_set_sensitive(colorsize_menu, FALSE);
+		gtk_widget_set_sensitive(menu_colorsize, FALSE);
 	} else {
-		gtk_widget_set_sensitive(colorsize_menu, TRUE);
+		gtk_widget_set_sensitive(menu_colorsize, TRUE);
 	}
 }
 
@@ -727,25 +772,60 @@ void insert_keyboard(const GtkWidget *widget) {
 	gtk_widget_show(scroll);
 
 	/* create the keyboard selection */
-	keymap = gtk_tree_view_new();
-	gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(keymap), FALSE);
-	gtk_widget_show(keymap);
+	tree_keymap = gtk_tree_view_new();
+	gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(tree_keymap), FALSE);
+	gtk_widget_show(tree_keymap);
 	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scroll),
-		keymap);
+		tree_keymap);
 
 	model = gtk_list_store_new(1, G_TYPE_STRING);
-	gtk_tree_view_set_model(GTK_TREE_VIEW(keymap), GTK_TREE_MODEL(model));
+	gtk_tree_view_set_model(GTK_TREE_VIEW(tree_keymap), GTK_TREE_MODEL(model));
 	loadKeymap(model);
 
 	cell = gtk_cell_renderer_text_new();
 	column = gtk_tree_view_column_new_with_attributes(_("Keyboard"), cell, "text", 0, NULL);
-	gtk_tree_view_append_column(GTK_TREE_VIEW(keymap),
+	gtk_tree_view_append_column(GTK_TREE_VIEW(tree_keymap),
 		GTK_TREE_VIEW_COLUMN(column));
 
-	gtk_signal_connect(GTK_OBJECT(keymap), "cursor-changed",
+	gtk_signal_connect(GTK_OBJECT(tree_keymap), "cursor-changed",
 		GTK_SIGNAL_FUNC(sig_keychange), NULL);
 }
 
+void insert_sound(const GtkWidget *widget) {
+	GtkWidget *menu;
+	gint i, count;
+
+	menu_sound = gtk_option_menu_new();
+	gtk_table_attach(GTK_TABLE(widget), menu_sound, 0, 1, 0, 1,
+		GTK_SHRINK, GTK_SHRINK, 0, 2);
+	gtk_widget_show(menu_sound);
+	menu = gtk_menu_new();
+
+	count = g_list_length(sound_options);
+	for(i=0; i<count; i++) {
+		GtkWidget *menuitem;
+		gchar *item;
+
+		item = g_list_nth_data(sound_options, i);
+		if(item == NULL)
+			return;
+
+		menuitem = gtk_menu_item_new_with_label(item);
+		gtk_widget_show(menuitem);
+		gtk_menu_append(GTK_MENU(menu), menuitem);
+		gtk_signal_connect(GTK_OBJECT(menuitem), "activate",
+			GTK_SIGNAL_FUNC(sig_sound), NULL);
+	}
+	gtk_option_menu_set_menu(GTK_OPTION_MENU(menu_sound), menu);
+
+	if(iSHASH("rdp_protocol") < 1) { /* disable, it's not supported by the RDP version */
+		gtk_widget_set_sensitive(menu_sound, FALSE);
+	} else {
+		gtk_widget_set_sensitive(menu_sound, TRUE);
+	}
+
+}
+
 void insert_program(const GtkWidget *widget) {
 	GtkWidget *lBox1, *lBox2;
 	GtkWidget *progLabel, *pathLabel;
@@ -767,13 +847,13 @@ void insert_program(const GtkWidget *widget) {
 	gtk_widget_show(progLabel);
 
 	/* create the program editbox */
-	program = gtk_entry_new();
-	gtk_entry_set_max_length(GTK_ENTRY(program), 255);
-	g_signal_connect(G_OBJECT(GTK_ENTRY(program)), "changed",
+	input_program = gtk_entry_new();
+	gtk_entry_set_max_length(GTK_ENTRY(input_program), 255);
+	g_signal_connect(G_OBJECT(GTK_ENTRY(input_program)), "changed",
 		G_CALLBACK(sig_program), NULL);
-	gtk_table_attach(GTK_TABLE(widget), program, 0, 1, 2, 3,
+	gtk_table_attach(GTK_TABLE(widget), input_program, 0, 1, 2, 3,
 		GTK_EXPAND|GTK_FILL, GTK_EXPAND|GTK_FILL, 0, 5);
-	gtk_widget_show(program);
+	gtk_widget_show(input_program);
 
 	lBox2 = gtk_hbox_new(FALSE, 0);
 	gtk_container_set_border_width(GTK_CONTAINER(lBox2), 0);
@@ -786,13 +866,13 @@ void insert_program(const GtkWidget *widget) {
 	gtk_widget_show(pathLabel);
 
 	/* create the program editbox */
-	ppath = gtk_entry_new();
-	gtk_entry_set_max_length(GTK_ENTRY(ppath), 255);
-	g_signal_connect(G_OBJECT(GTK_ENTRY(ppath)), "changed",
+	input_ppath = gtk_entry_new();
+	gtk_entry_set_max_length(GTK_ENTRY(input_ppath), 255);
+	g_signal_connect(G_OBJECT(GTK_ENTRY(input_ppath)), "changed",
 		G_CALLBACK(sig_path), NULL);
-	gtk_table_attach(GTK_TABLE(widget), ppath, 0, 1, 4, 5,
+	gtk_table_attach(GTK_TABLE(widget), input_ppath, 0, 1, 4, 5,
 		GTK_EXPAND|GTK_FILL, GTK_EXPAND|GTK_FILL, 0, 5);
-	gtk_widget_show(ppath);
+	gtk_widget_show(input_ppath);
 
 	/* make sure, the entry widget is init */
 	g_signal_connect(G_OBJECT(GTK_CHECK_BUTTON(check_prog)), "toggled",
@@ -801,8 +881,8 @@ void insert_program(const GtkWidget *widget) {
 
 void insert_settings(const GtkWidget *widget) {
 	GtkWidget *sep;
-	GtkWidget *cBox;
-	GtkWidget *clientnameLabel;
+	GtkWidget *cBox1, *cBox2;
+	GtkWidget *clientnameLabel, *sshhostLabel;
 
 	/* insert checkboxes */
 	check_bitmapupd = gtk_check_button_new_with_label(_("Force bitmap updates"));
@@ -863,23 +943,42 @@ void insert_settings(const GtkWidget *widget) {
 		GTK_EXPAND|GTK_FILL, GTK_EXPAND|GTK_FILL, 0, 5);
 	gtk_widget_show(sep);
 	
-	cBox = gtk_hbox_new(FALSE, 0);
-	gtk_container_set_border_width(GTK_CONTAINER(cBox), 0);
-	gtk_table_attach(GTK_TABLE(widget), cBox, 0, 1, 7, 8,
+	cBox1 = gtk_hbox_new(FALSE, 0);
+	gtk_container_set_border_width(GTK_CONTAINER(cBox1), 0);
+	gtk_table_attach(GTK_TABLE(widget), cBox1, 0, 1, 7, 8,
 		GTK_EXPAND|GTK_FILL, GTK_EXPAND|GTK_FILL, 0, 5);
-	gtk_widget_show(cBox);
+	gtk_widget_show(cBox1);
 
 	clientnameLabel = gtk_label_new(_("Fake clientname:"));
-	gtk_box_pack_start(GTK_BOX(cBox), clientnameLabel, FALSE, FALSE, FALSE);
+	gtk_box_pack_start(GTK_BOX(cBox1), clientnameLabel, FALSE, FALSE, FALSE);
 	gtk_widget_show(clientnameLabel);
 
-	clientname = gtk_entry_new();
-	gtk_entry_set_max_length(GTK_ENTRY(clientname), 255);
-	g_signal_connect(G_OBJECT(GTK_ENTRY(clientname)), "changed",
+	combo_clientname = gtk_combo_new();
+	gtk_entry_set_max_length(GTK_ENTRY(GTK_COMBO(combo_clientname)->entry), 255);
+	g_signal_connect(G_OBJECT(GTK_ENTRY(GTK_COMBO(combo_clientname)->entry)), "changed",
 		G_CALLBACK(sig_clientname), NULL);
-	gtk_table_attach(GTK_TABLE(widget), clientname, 0, 1, 8, 9,
+	gtk_table_attach(GTK_TABLE(widget), combo_clientname, 0, 1, 8, 9,
 		GTK_EXPAND|GTK_FILL, GTK_EXPAND|GTK_FILL, 0, 5);
-	gtk_widget_show(clientname);
+	gtk_widget_show(combo_clientname);
+
+	/* create the sshhost editbox */
+	cBox2 = gtk_hbox_new(FALSE, 0);
+	gtk_container_set_border_width(GTK_CONTAINER(cBox2), 0);
+	gtk_table_attach(GTK_TABLE(widget), cBox2, 0, 1, 9, 10,
+		GTK_EXPAND|GTK_FILL, GTK_EXPAND|GTK_FILL, 0, 5);
+	gtk_widget_show(cBox2);
+
+	sshhostLabel = gtk_label_new(_("SSH Tunnel-Host (User):"));
+	gtk_box_pack_start(GTK_BOX(cBox2), sshhostLabel, FALSE, FALSE, FALSE);
+	gtk_widget_show(sshhostLabel);
+
+	input_sshhost = gtk_entry_new();
+	gtk_entry_set_max_length(GTK_ENTRY(input_sshhost), 255);
+	g_signal_connect(G_OBJECT(GTK_ENTRY(input_sshhost)), "changed",
+		G_CALLBACK(sig_sshhost), NULL);
+	gtk_table_attach(GTK_TABLE(widget), input_sshhost, 0, 1, 10, 11,
+		GTK_EXPAND|GTK_FILL, GTK_EXPAND|GTK_FILL, 0, 5);
+	gtk_widget_show(input_sshhost);
 }
 
 void sig_username(GtkWidget *widget, gpointer data) {
@@ -946,11 +1045,12 @@ void sig_rdp_protocol(GtkWidget *widget, gpointer data) {
 	}
 
 	if(iSHASH("rdp_protocol") < 1) { /* disable, it's not supported by the RDP version */
-		gtk_widget_set_sensitive(colorsize_menu, FALSE);
+		gtk_widget_set_sensitive(menu_colorsize, FALSE);
 		gtk_widget_set_sensitive(check_attconsole, FALSE);
+		gtk_widget_set_sensitive(menu_sound, FALSE);
 	} else {
-		gtk_widget_set_sensitive(colorsize_menu, TRUE);
-		gtk_widget_set_sensitive(check_attconsole, TRUE);
+		gtk_widget_set_sensitive(menu_colorsize, TRUE);
+		gtk_widget_set_sensitive(menu_sound, TRUE);
 	}
 }
 
@@ -1024,13 +1124,13 @@ void sig_colchange(GtkWidget *widget, gpointer data) {
 	gint item;
 	gchar *filename;
 
-	item = gtk_option_menu_get_history(GTK_OPTION_MENU(colorsize_menu));
+	item = gtk_option_menu_get_history(GTK_OPTION_MENU(menu_colorsize));
 
 	filename = g_strdup_printf("%s/%s_%d.png", PIXDIR, "colors", item+1);
 	setBHASH("colorsize", item);
 
 	if(g_file_test(filename, G_FILE_TEST_EXISTS))
-		gtk_image_set_from_file(GTK_IMAGE(colorsize_img), filename);
+		gtk_image_set_from_file(GTK_IMAGE(image_colorsize), filename);
 
 	if(filename != NULL)
 		g_free(filename);
@@ -1065,6 +1165,22 @@ void sig_keychange(GtkWidget *widget, gpointer data) {
 		g_free(text);
 }
 
+void sig_sound(GtkWidget *widget, gpointer data) {
+	gint item;
+
+	item = gtk_option_menu_get_history(GTK_OPTION_MENU(menu_sound));
+
+	setBHASH("sound", item);
+
+	if(SHASH("sound") != NULL) {
+		g_hash_table_replace(config, "sound",
+			g_strdup_printf("%d", item));
+	} else {
+		g_hash_table_insert(config, "sound",
+			g_strdup_printf("%d", item));
+	}
+}
+
 void sig_program(GtkWidget *widget, gpointer data) {
 	if(SHASH("program") != NULL)
 		g_hash_table_replace(config, "program",
@@ -1100,8 +1216,8 @@ void sig_runprog(GtkWidget *widget, gpointer data) {
 			gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget))));
 	}
 
-	gtk_widget_set_sensitive(program, iSHASH("runprog"));
-	gtk_widget_set_sensitive(ppath, iSHASH("runprog"));
+	gtk_widget_set_sensitive(input_program, iSHASH("runprog"));
+	gtk_widget_set_sensitive(input_ppath, iSHASH("runprog"));
 }
 
 void sig_bitmapupdate(GtkWidget *widget, gpointer data) {
@@ -1188,3 +1304,15 @@ void sig_clientname(GtkWidget *widget, gpointer data) {
 		g_hash_table_remove(config, "clientname");
 }
 
+void sig_sshhost(GtkWidget *widget, gpointer data) {
+	if(SHASH("sshhost") != NULL)
+		g_hash_table_replace(config, "sshhost",
+			g_strdup(gtk_entry_get_text(GTK_ENTRY(widget))));
+	else
+		g_hash_table_insert(config, "sshhost",
+			g_strdup(gtk_entry_get_text(GTK_ENTRY(widget))));
+
+	if(SHASH("sshhost") == NULL)
+		g_hash_table_remove(config, "sshhost");
+}
+
